
% WHAT IF...? OUTPUT LAYER

% FIG 3A

opengl software
figure;

% Current signature

x = -0.7:0.05:20;

g1_amp = .985; % amplitude - increasing the value increases peak amplitude.
% Changing the amplitude doesn't change the rise time. It is only used to
% compensate for an effect on amplitude of dur.
g1_dur = 2.5; % duration - increasing the value increases the time constant
% of skewedgaussian (by a diminishing ratio i.e. a linear increase 
% of time constant requires an exponential increase of dur).

gaussian1 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g1_dur))*g1_amp)% a normal distribution
skewedgaussian1 = @(x,alpha) 2*gaussian1(x).*normcdf(alpha*x)% that normal 
% distribution skewed

g2_amp = .92;  
g2_dur = 8.6; 

gaussian2 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g2_dur))*g2_amp)% the same 
% distribution as gaussian1 but with SD adjusted by SDproxy
skewedgaussian2 = @(x,alpha) 2*gaussian2(x).*normcdf(alpha*x)% skewed

g3_amp = .89;
g3_dur = 31;

gaussian3 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g3_dur))*g3_amp)
skewedgaussian3 = @(x,alpha) 2*gaussian3(x).*normcdf(alpha*x)

g4_amp = .88;
g4_dur = 66.5;

gaussian4 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g4_dur))*g4_amp)
skewedgaussian4 = @(x,alpha) 2*gaussian4(x).*normcdf(alpha*x)

% g5_amp = .88;
% g5_dur = 74;
% 
% gaussian5 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g5_dur))*g5_amp)
% skewedgaussian5 = @(x,alpha) 2*gaussian5(x).*normcdf(alpha*x)

% DERIVE TIME CONSTANTS
g1_influxUnit = skewedgaussian1(x, 4); % an array of y values in the range for x, 
% set above, representing charge influx driven by a single spike
[M,I] = max(g1_influxUnit); % M is the peak value in array and I its index position
drop = M*.368; % a 63.2% reduction of M
[~,closestIndex] = min(abs(drop-g1_influxUnit(I:end))); % Finds the index position 
% of the nearest value to 'drop' in the array 'I:end'
g1_timeConstant = closestIndex/20 % There are 20 index positions per ms, so 
% dividing the distance (measured in index positions) between I and
% closestIndex by 20 gives the time interval between the peak value of
% the skewed distribution and a reduction by 63.2%

g2_influxUnit = skewedgaussian2(x, 4);
[M,I] = max(g2_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g2_influxUnit(I:end)));
g2_timeConstant = closestIndex/20

g3_influxUnit = skewedgaussian3(x, 4);
[M,I] = max(g3_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g3_influxUnit(I:end)));
g3_timeConstant = closestIndex/20

g4_influxUnit = skewedgaussian4(x, 4);
[M,I] = max(g4_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g4_influxUnit(I:end)));
g4_timeConstant = closestIndex/20

% g5_influxUnit = skewedgaussian5(x, 4);
% [M,I] = max(g5_influxUnit);
% drop = M*.368;
% [~,closestIndex] = min(abs(drop-g5_influxUnit(I:end)));
% g5_timeConstant = closestIndex/20

% Some variables

Hz = 50; % PC firing rate
P = Hz/20000; % The probability of a spike in any moment (a moment being 
% 1/20th of a ms).
Q = 60; % The number of simulated PCs afferent to a single nuclear cell
R = 20; % Refractory period: 20 = 1 ms
sites = 40; % The number of regions of the nuclear soma surface into which it 
% is subdivided to simulate the computational effect of random sampling by
% each region of PCs afferent to the cell.  
samplesize = 30; % Sample size.

% NOTE: Q, sites and S incorporate into the simulation the fact that a nuclear 
% cell receives contact from PCs in addition to the number that make
% 'significant' contact, and also variability of the number of boutons per
% PC and of the permutation of PCs that make contact on each region

% DATA FOR 1.25 ms TIME CONSTANT 

j = 1;
master = zeros([1 4000]); % 4,000 1/20ths of a ms, i.e., 200 ms

while j < Q + 1

    PCspikes = zeros([1 4400]); % 200 ms plus an extra 20 ms so that if 
    % charge influx is triggered towards the end of the modelled period, 
    % the current signature is not truncated
    
    for i = 1:4400
        r = rand;
        if r <= P
            PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
            % charge influx
        end
    end
    
    PCcurrent = zeros([1 4400]);
    
    i = 1;
    while i < 4400
        if i < length(PCspikes)-(length(g1_influxUnit)-1)
            if PCspikes(i) == 1
                PCcurrent(i:i+length(g1_influxUnit)-1) = ...
                    PCcurrent(i:i+length(g1_influxUnit)-1)+g1_influxUnit;
                i = i + R; % represents refractory period
            else
                i = i + 1;
            end
        else
            i = i + 1;
        end
    end

    PCcurrent = PCcurrent(1:4000);

    if j == 1
        master = master + PCcurrent; % An array
    else
        master = [master; PCcurrent]; % Builds into a Q by 4,000 matrix
    end

    j = j + 1;

end

k = 1;
remastered = zeros(sites,4000);

while k < sites + 1
    m = randperm(Q);
    newmaster = master(m(1:samplesize),:);
    newmaster = sum(newmaster);
    remastered(k,:) = newmaster;
    k = k + 1;
end

mu = mean(remastered);

% subplot(2,10,[1 2 3 4 5])
subplot(2,1,1)
x = 1:4000;
plot(x,remastered(1,:),'Color',([0 0 .6]))%,'LineWidth',1.4)
xlabel('ms'); ylabel('current'); title('10 samples of 30 out of 60 PCs, 50 Hz');
xlim([0 4000]); xticks([0 500 1000 1500 2000 2500 3000 3500 4000]); 
xticklabels([0 25 50 75 100 125 150 175 200])
ylim([0 12.6]); yticks([0 2.8 5.6 8.4 11.2]);
yticklabels([0 4 8 12 16])
set(gca,'FontSize',10)

hold on

% plot(x,remastered(2,:),'Color',([0 0 .6]))

k = 2;

while k < (sites/4) + 1
    plot(x,remastered(k,:),'Color',([0 0 .6]))%,'LineWidth',1.4)
    k = k + 1;
end

plot(x,mu,'Color',([0 0 .6]),'LineWidth',1.2)

% DATA FOR 2.5 ms TIME CONSTANT 

j = 1;
master = zeros([1 4000]); % 4,000 1/20ths of a ms, i.e., 200 ms

while j < Q + 1

    PCspikes = zeros([1 4400]); % 200 ms plus an extra 20 ms so that if 
    % charge influx is triggered towards the end of the modelled period, 
    % the current signature is not truncated
    
    for i = 1:4400
        r = rand;
        if r <= P
            PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
            % charge influx
        end
    end
    
    PCcurrent = zeros([1 4400]);
    
    i = 1;
    while i < 4400
        if i < length(PCspikes)-(length(g2_influxUnit)-1)
            if PCspikes(i) == 1
                PCcurrent(i:i+length(g2_influxUnit)-1) = ...
                    PCcurrent(i:i+length(g2_influxUnit)-1)+g2_influxUnit;
                i = i + R; % represents refractory period
            else
                i = i + 1;
            end
        else
            i = i + 1;
        end
    end

    PCcurrent = PCcurrent(1:4000);

    if j == 1
        master = master + PCcurrent; % An array
    else
        master = [master; PCcurrent]; % Builds into a Q by 4,000 matrix
    end

    j = j + 1;

end

k = 1;
remastered = zeros(sites,4000);

while k < sites + 1
    m = randperm(Q);
    newmaster = master(m(1:samplesize),:);
    newmaster = sum(newmaster);
    remastered(k,:) = newmaster;
    k = k + 1;
end

mu = mean(remastered);

k = 1;

while k < (sites/4) + 1
    plot(x,remastered(k,:),'Color',([1 .84 0]))%,'LineWidth',1.4)
    k = k + 1;
end

plot(x,mu,'Color',([1 .74 0]),'LineWidth',1.2)

% DATA FOR 5 ms TIME CONSTANT 

j = 1;
master = zeros([1 4000]); % 4,000 1/20ths of a ms, i.e., 200 ms

while j < Q + 1

    PCspikes = zeros([1 4400]); % 200 ms plus an extra 20 ms so that if 
    % charge influx is triggered towards the end of the modelled period, 
    % the current signature is not truncated
    
    for i = 1:4400
        r = rand;
        if r <= P
            PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
            % charge influx
        end
    end
    
    PCcurrent = zeros([1 4400]);
    
    i = 1;
    while i < 4400
        if i < length(PCspikes)-(length(g3_influxUnit)-1)
            if PCspikes(i) == 1
                PCcurrent(i:i+length(g3_influxUnit)-1) = ...
                    PCcurrent(i:i+length(g3_influxUnit)-1)+g3_influxUnit;
                i = i + R; % represents refractory period
            else
                i = i + 1;
            end
        else
            i = i + 1;
        end
    end

    PCcurrent = PCcurrent(1:4000);

    if j == 1
        master = master + PCcurrent; % An array
    else
        master = [master; PCcurrent]; % Builds into a Q by 4,000 matrix
    end

    j = j + 1;

end

k = 1;
remastered = zeros(sites,4000);

while k < sites + 1
    m = randperm(Q);
    newmaster = master(m(1:samplesize),:);
    newmaster = sum(newmaster);
    remastered(k,:) = newmaster;
    k = k + 1;
end

mu = mean(remastered);

k = 1;

while k < (sites/4) + 1
    plot(x,remastered(k,:),'Color',([1 .5 .5]))%,'LineWidth',1.4)
    k = k + 1;
end

plot(x,mu,'Color',([1 .5 .5]),'LineWidth',1.2)

% DATA FOR 7.5 ms TIME CONSTANT 

j = 1;
master = zeros([1 4000]); % 4,000 1/20ths of a ms, i.e., 200 ms

while j < Q + 1

    PCspikes = zeros([1 4400]); % 200 ms plus an extra 20 ms so that if 
    % charge influx is triggered towards the end of the modelled period, 
    % the current signature is not truncated
    
    for i = 1:4400
        r = rand;
        if r <= P
            PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
            % charge influx
        end
    end
    
    PCcurrent = zeros([1 4400]);
    
    i = 1;
    while i < 4400
        if i < length(PCspikes)-(length(g4_influxUnit)-1)
            if PCspikes(i) == 1
                PCcurrent(i:i+length(g4_influxUnit)-1) = ...
                    PCcurrent(i:i+length(g4_influxUnit)-1)+g4_influxUnit;
                i = i + R; % represents refractory period
            else
                i = i + 1;
            end
        else
            i = i + 1;
        end
    end

    PCcurrent = PCcurrent(1:4000);

    if j == 1
        master = master + PCcurrent; % An array
    else
        master = [master; PCcurrent]; % Builds into a Q by 4,000 matrix
    end

    j = j + 1;

end

k = 1;
remastered = zeros(sites,4000);

while k < sites + 1
    m = randperm(Q);
    newmaster = master(m(1:samplesize),:);
    newmaster = sum(newmaster);
    remastered(k,:) = newmaster;
    k = k + 1;
end

mu = mean(remastered);

k = 1;

while k < (sites/4) + 1
    plot(x,remastered(k,:),'Color',([.6 .6 .6]))%,'LineWidth',1.4)
    k = k + 1;
end

plot(x,mu,'Color',([.6 .6 .6]),'LineWidth',1.2)
