
% WHAT IF...? OUTPUT LAYER

% FIG 3BtoD

opengl software
figure;

% Current signature

x = -0.7:0.05:20;

g1_amp = .985; % amplitude - increasing the value increases peak amplitude.
% Changing the amplitude doesn't change the rise time. It is only used to
% compensate for an effect on amplitude of dur.
g1_dur = 2.5; % duration - increasing the value increases the time constant
% of skewedgaussian (by a diminishing ratio i.e. a linear increase 
% of time constant requires an exponential increase of dur).

gaussian1 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g1_dur))*g1_amp)% a normal distribution
skewedgaussian1 = @(x,alpha) 2*gaussian1(x).*normcdf(alpha*x)% that normal 
% distribution skewed

g2_amp = .92;  
g2_dur = 8.6; 

gaussian2 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g2_dur))*g2_amp)% the same 
% distribution as gaussian1 but with SD adjusted by SDproxy
skewedgaussian2 = @(x,alpha) 2*gaussian2(x).*normcdf(alpha*x)% skewed

g3_amp = .89;
g3_dur = 31;

gaussian3 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g3_dur))*g3_amp)
skewedgaussian3 = @(x,alpha) 2*gaussian3(x).*normcdf(alpha*x)

g4_amp = .88;
g4_dur = 66.5;

gaussian4 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g4_dur))*g4_amp)
skewedgaussian4 = @(x,alpha) 2*gaussian4(x).*normcdf(alpha*x)

% g5_amp = .88;
% g5_dur = 74;
% 
% gaussian5 = @(x) ((1/sqrt((2*pi))*exp(-x.^2/g5_dur))*g5_amp)
% skewedgaussian5 = @(x,alpha) 2*gaussian5(x).*normcdf(alpha*x)

% DERIVE TIME CONSTANTS
g1_influxUnit = skewedgaussian1(x, 4); % an array of y values in the range for x, 
% set above, representing charge influx driven by a single spike
[M,I] = max(g1_influxUnit); % M is the peak value in array and I its index position
drop = M*.368; % a 63.2% reduction of M
[~,closestIndex] = min(abs(drop-g1_influxUnit(I:end))); % Finds the index position 
% of the nearest value to 'drop' in the array 'I:end'
g1_timeConstant = closestIndex/20 % There are 20 index positions per ms, so 
% dividing the distance (measured in index positions) between I and
% closestIndex by 20 gives the time interval between the peak value of
% the skewed distribution and a reduction by 63.2%

g2_influxUnit = skewedgaussian2(x, 4);
[M,I] = max(g2_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g2_influxUnit(I:end)));
g2_timeConstant = closestIndex/20

g3_influxUnit = skewedgaussian3(x, 4);
[M,I] = max(g3_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g3_influxUnit(I:end)));
g3_timeConstant = closestIndex/20

g4_influxUnit = skewedgaussian4(x, 4);
[M,I] = max(g4_influxUnit);
drop = M*.368;
[~,closestIndex] = min(abs(drop-g4_influxUnit(I:end)));
g4_timeConstant = closestIndex/20

% g5_influxUnit = skewedgaussian5(x, 4);
% [M,I] = max(g5_influxUnit);
% drop = M*.368;
% [~,closestIndex] = min(abs(drop-g5_influxUnit(I:end)));
% g5_timeConstant = closestIndex/20

% Some variables

Hz = 50; % PC firing rate
P = Hz/20000; % The probability of a spike in any moment (a moment being 
% 1/20th of a ms).
Q = 60; % The number of simulated PCs afferent to a single nuclear cell
R = 20; % Refractory period: 20 = 1 ms
sites = 40; % The number of regions of the nuclear soma surface into which it 
% is subdivided to simulate the computational effect of random sampling by
% each region of PCs afferent to the cell.  
samplesize = 30; % Sample size.

% NOTE: Q, sites and S incorporate into the simulation the fact that a nuclear 
% cell receives contact from PCs in addition to the number that make
% 'significant' contact, and also variability of the number of boutons per
% PC and of the permutation of PCs that make contact on each region

%%% MU %%%

maxHz = 200; % Max PC firing rate  

% DATA FOR 1.25 ms TIME CONSTANT 

cycle = 1

masterMU = zeros([1 maxHz]);
masterSIG1 = zeros([1 maxHz]);
masterRSD1 = zeros([1 maxHz]);

Hz = 1; % PC firing rate

while Hz < maxHz + 1

    P = Hz/20000; % The probability of a spike in any moment (a moment being 
    % 1/20th of a ms.
    
    j = 1;
    master = zeros([1 4000]); % 4,000 1/20ths of a ms - i.e., 200 ms, plus an 
    % extra 20 ms so that if charge influx is triggered towards the end of the
    % modelled period, the current signature is not truncated

    while j < Q + 1
    
        PCspikes = zeros([1 4400]);
        
        for i = 1:4400
            r = rand;
            if r <= P
                PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
                % charge influx
            end
        end
        
        PCcurrent = zeros([1 4400]);
        
        i = 1;
        while i < 4400
            if i < length(PCspikes)-(length(g1_influxUnit)-1)
                if PCspikes(i) == 1
                    PCcurrent(i:i+length(g1_influxUnit)-1) = ...
                        PCcurrent(i:i+length(g1_influxUnit)-1)+g1_influxUnit;
                    i = i + R; % represents refractory period
                else
                    i = i + 1;
                end
            else
                i = i + 1;
            end
        end
    
        PCcurrent = PCcurrent(1:4000);
    
        if j == 1
            master = master + PCcurrent; % An array
        else
            master = [master; PCcurrent]; % Builds into a Q by 4,000 matrix
        end
    
        j = j + 1;
    
    end

    k = 1;
    remastered = zeros(sites,4000);
    
    while k < sites + 1
        m = randperm(Q); % an array containing the integers 1 to Q in a 
        % random order
        newmaster = master(m(1:samplesize),:); % a samplesize by 4,000 
        % matrix containing randomly selected rows of master
        newmaster = sum(newmaster); % a 1 by 4,000 array
        remastered(k,:) = newmaster; % newmaster becomes row k of 
        % remastered. remastered is a sites by 4,000 array populated in
        % this way
        k = k + 1;
    end

% master = master(1:4000);
% remastered = integrate_all_intervals(master);

mu = mean(remastered); % a 1 by 4,000 array populated by the mean of each 
% column of remastered. This represents the rate of charge entry to the
% nuclear soma averaged across all sites (of the soma surface) at each 
% 1/20 ms timestep, for 200 ms
mu = integrate_all_intervals(mu);
MU = mean(mu); % a single value, the mean of mu, i.e., the mean rate of 
% charge entry in a 200 ms period
masterMU(Hz) = MU;

SIG = std(mu);
masterSIG1(Hz) = SIG;

RSD = SIG/MU;
masterRSD1(Hz) = RSD;

Hz = Hz + 1;

end

subplot(2,6,5)
x = 1:maxHz;
plot(x,masterMU,'Color',([0 0 .6]),'LineWidth',1)
xlabel('PC Hz'); ylabel('current'); title('mu, 1 ms')
xlim([50 200]); 
ylim([0 28]); yticks([0 7 14 21 28]);
yticklabels([0 10 20 30 40])
set(gca,'FontSize',10)

hold on

% DATA FOR 2.5 ms TIME CONSTANT 

cycle = 2

masterMU = zeros([1 maxHz]);
masterSIG2 = zeros([1 maxHz]);
masterRSD2 = zeros([1 maxHz]);

Hz = 1; % PC firing rate

while Hz < maxHz + 1

    P = Hz/20000; % The probability of a spike in any moment (a moment being 
    % 1/20th of a ms.
    
    j = 1;
    master = zeros([1 4000]); % 4,000 1/20ths of a ms - i.e., 200 ms, plus an 
    % extra 20 ms so that if charge influx is triggered towards the end of the
    % modelled period, the current signature is not truncated

    while j < Q + 1
    
        PCspikes = zeros([1 4400]);
        
        for i = 1:4400
            r = rand;
            if r <= P
                PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
                % charge influx
            end
        end
        
        PCcurrent = zeros([1 4400]);
        
        i = 1;
        while i < 4400
            if i < length(PCspikes)-(length(g2_influxUnit)-1)
                if PCspikes(i) == 1
                    PCcurrent(i:i+length(g2_influxUnit)-1) = ...
                        PCcurrent(i:i+length(g2_influxUnit)-1)+g2_influxUnit;
                    i = i + R; % represents refractory period
                else
                    i = i + 1;
                end
            else
                i = i + 1;
            end
        end
    
        PCcurrent = PCcurrent(1:4000);
    
        if j == 1
            master = master + PCcurrent; % An array
        else
            master = [master; PCcurrent]; % Builds into a Q by 4,000 matrix
        end
    
        j = j + 1;
    
    end

    k = 1;
    remastered = zeros(sites,4000);
    
    while k < sites + 1
        m = randperm(Q); % an array containing the integers 1 to Q in a 
        % random order
        newmaster = master(m(1:samplesize),:); % a samplesize by 4,000 
        % matrix containing randomly selected rows of master
        newmaster = sum(newmaster); % a 1 by 4,000 array
        remastered(k,:) = newmaster; % newmaster becomes row k of 
        % remastered. remastered is a sites by 4,000 array populated in
        % this way
        k = k + 1;
    end

% master = master(1:4000);
% remastered = integrate_all_intervals(master);

mu = mean(remastered); % a 1 by 4,000 array populated by the mean of each 
% column of remastered. This represents the rate of charge entry to the
% nuclear soma averaged across all sites (of the soma surface) at each 
% 1/20 ms timestep, for 200 ms
mu = integrate_all_intervals(mu);
MU = mean(mu); % a single value, the mean of mu, i.e., the mean rate of 
% charge entry in a 200 ms period
masterMU(Hz) = MU;

SIG = std(mu);
masterSIG2(Hz) = SIG;

RSD = SIG/MU;
masterRSD2(Hz) = RSD;

Hz = Hz + 1;

end

plot(x,masterMU,'Color',([1 .75 0]),'LineWidth',1)

% DATA FOR 5 ms TIME CONSTANT 

cycle = 3

masterMU = zeros([1 maxHz]);
masterSIG3 = zeros([1 maxHz]);
masterRSD3 = zeros([1 maxHz]);

Hz = 1; % PC firing rate

while Hz < maxHz + 1

    P = Hz/20000; % The probability of a spike in any moment (a moment being 
    % 1/20th of a ms.
    
    j = 1;
    master = zeros([1 4000]); % 4,000 1/20ths of a ms - i.e., 200 ms, plus an 
    % extra 20 ms so that if charge influx is triggered towards the end of the
    % modelled period, the current signature is not truncated

    while j < Q + 1
    
        PCspikes = zeros([1 4400]);
        
        for i = 1:4400
            r = rand;
            if r <= P
                PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
                % charge influx
            end
        end
        
        PCcurrent = zeros([1 4400]);
        
        i = 1;
        while i < 4400
            if i < length(PCspikes)-(length(g3_influxUnit)-1)
                if PCspikes(i) == 1
                    PCcurrent(i:i+length(g3_influxUnit)-1) = ...
                        PCcurrent(i:i+length(g3_influxUnit)-1)+g3_influxUnit;
                    i = i + R; % represents refractory period
                else
                    i = i + 1;
                end
            else
                i = i + 1;
            end
        end
    
        PCcurrent = PCcurrent(1:4000);
    
        if j == 1
            master = master + PCcurrent; % An array
        else
            master = [master; PCcurrent]; % Builds into a Q by 4,000 matrix
        end
    
        j = j + 1;
    
    end

    k = 1;
    remastered = zeros(sites,4000);
    
    while k < sites + 1
        m = randperm(Q); % an array containing the integers 1 to Q in a 
        % random order
        newmaster = master(m(1:samplesize),:); % a samplesize by 4,000 
        % matrix containing randomly selected rows of master
        newmaster = sum(newmaster); % a 1 by 4,000 array
        remastered(k,:) = newmaster; % newmaster becomes row k of 
        % remastered. remastered is a sites by 4,000 array populated in
        % this way
        k = k + 1;
    end

% master = master(1:4000);
% remastered = integrate_all_intervals(master);

mu = mean(remastered); % a 1 by 4,000 array populated by the mean of each 
% column of remastered. This represents the rate of charge entry to the
% nuclear soma averaged across all sites (of the soma surface) at each 
% 1/20 ms timestep, for 200 ms
mu = integrate_all_intervals(mu);
MU = mean(mu); % a single value, the mean of mu, i.e., the mean rate of 
% charge entry in a 200 ms period
masterMU(Hz) = MU;

SIG = std(mu);
masterSIG3(Hz) = SIG;

RSD = SIG/MU;
masterRSD3(Hz) = RSD;

Hz = Hz + 1;

end

plot(x,masterMU,'Color',([1 .5 .5]),'LineWidth',1)

% DATA FOR 7.5 ms TIME CONSTANT 

cycle = 4

masterMU = zeros([1 maxHz]);
masterSIG4 = zeros([1 maxHz]);
masterRSD4 = zeros([1 maxHz]);

Hz = 1; % PC firing rate

while Hz < maxHz + 1

    P = Hz/20000; % The probability of a spike in any moment (a moment being 
    % 1/20th of a ms.
    
    j = 1;
    master = zeros([1 4000]); % 4,000 1/20ths of a ms - i.e., 200 ms, plus an 
    % extra 20 ms so that if charge influx is triggered towards the end of the
    % modelled period, the current signature is not truncated

    while j < Q + 1
    
        PCspikes = zeros([1 4400]);
        
        for i = 1:4400
            r = rand;
            if r <= P
                PCspikes(i) = 1; % a 1 represents an afferent spike, triggering 
                % charge influx
            end
        end
        
        PCcurrent = zeros([1 4400]);
        
        i = 1;
        while i < 4400
            if i < length(PCspikes)-(length(g4_influxUnit)-1)
                if PCspikes(i) == 1
                    PCcurrent(i:i+length(g4_influxUnit)-1) = ...
                        PCcurrent(i:i+length(g4_influxUnit)-1)+g4_influxUnit;
                    i = i + R; % represents refractory period
                else
                    i = i + 1;
                end
            else
                i = i + 1;
            end
        end
    
        PCcurrent = PCcurrent(1:4000);
    
        if j == 1
            master = master + PCcurrent; % An array
        else
            master = [master; PCcurrent]; % Builds into a Q by 4,000 matrix
        end
    
        j = j + 1;
    
    end

    k = 1;
    remastered = zeros(sites,4000);
    
    while k < sites + 1
        m = randperm(Q); % an array containing the integers 1 to Q in a 
        % random order
        newmaster = master(m(1:samplesize),:); % a samplesize by 4,000 
        % matrix containing randomly selected rows of master
        newmaster = sum(newmaster); % a 1 by 4,000 array
        remastered(k,:) = newmaster; % newmaster becomes row k of 
        % remastered. remastered is a sites by 4,000 array populated in
        % this way
        k = k + 1;
    end

% master = master(1:4000);
% remastered = integrate_all_intervals(master);

mu = mean(remastered); % a 1 by 4,000 array populated by the mean of each 
% column of remastered. This represents the rate of charge entry to the
% nuclear soma averaged across all sites (of the soma surface) at each 
% 1/20 ms timestep, for 200 ms
mu = integrate_all_intervals(mu);
MU = mean(mu); % a single value, the mean of mu, i.e., the mean rate of 
% charge entry in a 200 ms period
masterMU(Hz) = MU;

SIG = std(mu);
masterSIG4(Hz) = SIG;

RSD = SIG/MU;
masterRSD4(Hz) = RSD;

Hz = Hz + 1

end

plot(x,masterMU,'Color',([.6 .6 .6]),'LineWidth',1)

%%% RSD %%%

subplot(2,6,6)

masterRSD1 = masterRSD1*100;
masterRSD2 = masterRSD2*100;
masterRSD3 = masterRSD3*100;
masterRSD4 = masterRSD4*100;

x = 1:maxHz;
plot(x(5:5:end),masterRSD1(5:5:end),'Color',([0 0 .6]),'LineWidth',1)
xlabel('PC Hz'); ylabel('%'); title('CV, 1 ms')
xlim([50 200]); ylim([0 50]); 
yticks([0 10 20 30 40 50]); yticklabels([0 10 20 30 40 50])
set(gca,'FontSize',10)

hold on

plot(x(5:5:end),masterRSD2(5:5:end),'Color',([1 .75 0]),'LineWidth',1)
plot(x(5:5:end),masterRSD3(5:5:end),'Color',([1 .5 .5]),'LineWidth',1)
plot(x(5:5:end),masterRSD4(5:5:end),'Color',([.6 .6 .6]),'LineWidth',1)

